@@ -41,6 +41,7 @@
 
 #include "gmenu-tree.h"
 
+#include "gs-debug.h"
 #include "gs-job.h"
 
 #include "subprocs.h"
@@ -68,12 +69,6 @@ struct GSJobPrivate
         guint           watch_id;
 
         char           *current_theme;
-
-        gboolean        themes_valid;
-        GMenuTree      *themes_tree;
-
-        long            last_stat_time;
-        GList          *dir_mtimes;
 };
 
 typedef struct 
@@ -84,7 +79,7 @@ typedef struct
 
 static GObjectClass *parent_class = NULL;
 
-G_DEFINE_TYPE (GSJob, gs_job, G_TYPE_OBJECT);
+G_DEFINE_TYPE (GSJob, gs_job, G_TYPE_OBJECT)
 
 static const char *known_engine_locations [] = {
         SAVERDIR,
@@ -110,8 +105,9 @@ find_command (const char *command)
                 path = g_build_filename (known_engine_locations [i], command, NULL);
 
                 if (g_file_test (path, G_FILE_TEST_IS_EXECUTABLE)
-                    && ! g_file_test (path, G_FILE_TEST_IS_DIR))
+                    && ! g_file_test (path, G_FILE_TEST_IS_DIR)) {
                         return path;
+                }
 
                 g_free (path);
         }
@@ -129,6 +125,8 @@ check_command (char *command)
 
         g_shell_parse_argv (command, NULL, &argv, NULL);
         path = find_command (argv [0]);
+        g_strfreev (argv);
+
         if (path) {
                 g_free (path);
                 return TRUE;
@@ -138,72 +136,48 @@ check_command (char *command)
 }
 
 static void
-load_themes (GSJob *job)
+add_known_engine_locations_to_path (void)
 {
-        GTimeVal tv;
+        static gboolean already_added;
+        int      i;
+        GString *str;
 
-        if (job->priv->themes_tree) {
-                gmenu_tree_unref (job->priv->themes_tree);
+        /* We only want to add the items to the path once */
+        if (already_added)
+                return;
+
+        already_added = TRUE;
+
+        /* TODO: set a default PATH ? */
+
+        str = g_string_new (g_getenv ("PATH"));
+        for (i = 0; known_engine_locations [i]; i++) {
+                /* TODO: check that permissions are safe */
+                if (g_file_test (known_engine_locations [i], G_FILE_TEST_IS_DIR)) {
+                        g_string_append_printf (str, ":%s", known_engine_locations [i]);
+                }
         }
 
-	job->priv->themes_tree = gmenu_tree_lookup ("gnome-screensavers.menu", GMENU_TREE_FLAGS_NONE);
-
-        job->priv->themes_valid = TRUE;
-  
-        g_get_current_time (&tv);
-        job->priv->last_stat_time = tv.tv_sec;
+        g_setenv ("PATH", str->str, TRUE);
+        g_string_free (str, TRUE);
 }
 
-static gboolean
-gs_job_theme_rescan_if_needed (GSJob *job)
+static GMenuTree *
+get_themes_tree (void)
 {
-        ThemeDirMtime *dir_mtime;
-        GList         *d;
-        int            stat_res;
-        struct stat    stat_buf;
-        GTimeVal       tv;
+        static GMenuTree *themes_tree;
 
-        g_return_val_if_fail (GS_IS_JOB (job), FALSE);
-
-        for (d = job->priv->dir_mtimes; d != NULL; d = d->next) {
-                dir_mtime = d->data;
-
-                stat_res = g_lstat (dir_mtime->dir, &stat_buf);
-
-                /* dir mtime didn't change */
-                if (stat_res == 0 && 
-                    S_ISDIR (stat_buf.st_mode) &&
-                    dir_mtime->mtime == stat_buf.st_mtime)
-                        continue;
-                /* didn't exist before, and still doesn't */
-                if (dir_mtime->mtime == 0 &&
-                    (stat_res != 0 || !S_ISDIR (stat_buf.st_mode)))
-                        continue;
-	  
-                /*do_theme_change (job);*/
-                return TRUE;
+        if (themes_tree) {
+                return themes_tree;
         }
-  
-        g_get_current_time (&tv);
-        job->priv->last_stat_time = tv.tv_sec;
 
-        return FALSE;
-}
+        /* we only need to add the locations to the path once
+           and since this is only run once we'll do it here */
+        add_known_engine_locations_to_path ();
 
-static void
-ensure_valid_themes (GSJob *job)
-{
-        GTimeVal tv;
-  
-        if (job->priv->themes_valid) {
-                g_get_current_time (&tv);
+	themes_tree = gmenu_tree_lookup ("gnome-screensavers.menu", GMENU_TREE_FLAGS_NONE);
 
-                if (ABS (tv.tv_sec - job->priv->last_stat_time) > 5)
-                        gs_job_theme_rescan_if_needed (job);
-        }
-  
-        if (! job->priv->themes_valid)
-                load_themes (job);
+        return themes_tree;
 }
 
 GSJobThemeInfo *
@@ -289,8 +263,9 @@ find_info_for_id (GMenuTree  *tree,
         GSList             *l;
 
 	root = gmenu_tree_get_root_directory (tree);
-        if (! root)
+        if (! root) {
                 return NULL;
+        }
 
         items = gmenu_tree_directory_get_contents (root);
 
@@ -312,6 +287,7 @@ find_info_for_id (GMenuTree  *tree,
         }
 
         g_slist_free (items);
+        gmenu_tree_item_unref (root);
 
         return info;
 }
@@ -322,14 +298,15 @@ gs_job_lookup_theme_info (GSJob      *job,
 {
         GSJobThemeInfo *info;
         char           *id;
+        GMenuTree      *tree;
 
         g_return_val_if_fail (GS_IS_JOB (job), NULL);
         g_return_val_if_fail (name != NULL, NULL);
 
-        ensure_valid_themes (job);
+        tree = get_themes_tree ();
 
         id = g_strdup_printf ("%s.desktop", name);
-        info = find_info_for_id (job->priv->themes_tree, id);
+        info = find_info_for_id (tree, id);
         g_free (id);
 
         return info;
@@ -382,14 +359,17 @@ GSList *
 gs_job_get_theme_info_list (GSJob *job)
 {
         GSList             *l = NULL;
-	GMenuTreeDirectory *root;
+        GMenuTreeDirectory *root;
+        GMenuTree          *tree;
 
         g_return_val_if_fail (GS_IS_JOB (job), NULL);
 
-        ensure_valid_themes (job);
+        tree = get_themes_tree ();
+        root = gmenu_tree_get_root_directory (tree);
 
-	if ((root = gmenu_tree_get_root_directory (job->priv->themes_tree))) {
-		make_theme_list (&l, root, "gnome-screensavers.menu");
+        if (root) {
+                make_theme_list (&l, root, "gnome-screensavers.menu");
+                gmenu_tree_item_unref (root);
         }
 
         return l;
@@ -419,33 +399,10 @@ gs_job_class_init (GSJobClass *klass)
         g_type_class_add_private (klass, sizeof (GSJobPrivate));
 }
 
-static void
-add_known_engine_locations_to_path (void)
-{
-        int      i;
-        GString *str;
-
-        /* TODO: set a default PATH ? */
-
-        str = g_string_new (g_getenv ("PATH"));
-        for (i = 0; known_engine_locations [i]; i++) {
-                /* TODO: check that permissions are safe */
-                if (g_file_test (known_engine_locations [i], G_FILE_TEST_IS_DIR))
-                        g_string_append_printf (str, ":%s", known_engine_locations [i]);
-        }
-
-        g_setenv ("PATH", str->str, TRUE);
-        g_string_free (str, TRUE);
-}
-
 static void
 gs_job_init (GSJob *job)
 {
         job->priv = GS_JOB_GET_PRIVATE (job);
-
-        add_known_engine_locations_to_path ();
-
-        job->priv->themes_valid = FALSE;
 }
 
 /* adapted from gspawn.c */
@@ -456,12 +413,13 @@ wait_on_child (int pid)
 
  wait_again:
         if (waitpid (pid, &status, 0) < 0) {
-                if (errno == EINTR)
+                if (errno == EINTR) {
                         goto wait_again;
-                else if (errno == ECHILD)
+                } else if (errno == ECHILD) {
                         ; /* do nothing, child already reaped */
-                else
+                } else {
                         g_warning ("waitpid () should not fail in 'GSJob'");
+                }
         }
 
         return status;
@@ -507,9 +465,6 @@ gs_job_finalize (GObject *object)
         g_free (job->priv->current_theme);
         job->priv->current_theme = NULL;
 
-        if (job->priv->themes_tree)
-                gmenu_tree_unref (job->priv->themes_tree);
-
         G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -520,7 +475,15 @@ gs_job_set_widget  (GSJob     *job,
         g_return_if_fail (job != NULL);
         g_return_if_fail (GS_IS_JOB (job));
 
-        job->priv->widget = widget;
+        if (widget != job->priv->widget) {
+                job->priv->widget = widget;
+
+                /* restart job */
+                if (gs_job_is_running (job)) {
+                        gs_job_stop (job);
+                        gs_job_start (job);
+                }
+        }
 }
 
 gboolean
@@ -554,8 +517,9 @@ gs_job_set_theme  (GSJob      *job,
 
         g_free (job->priv->current_theme);
 
-        if (theme)
+        if (theme) {
                 job->priv->current_theme = g_strdup (theme);
+        }
 
         return TRUE;
 }
@@ -588,10 +552,12 @@ nice_process (int pid,
 {
         g_return_if_fail (pid > 0);
 
-        if (nice_level == 0)
+        if (nice_level == 0) {
                 return;
+        }
 
 #if defined(HAVE_SETPRIORITY) && defined(PRIO_PROCESS)
+        gs_debug ("Setting child process priority to: %d", nice_level);
         if (setpriority (PRIO_PROCESS, pid, nice_level) != 0) {
                 g_warning ("setpriority(PRIO_PROCESS, %lu, %d) failed",
                            (unsigned long) pid, nice_level);
@@ -601,6 +567,51 @@ nice_process (int pid,
 #endif
 }
 
+static GPtrArray *
+get_env_vars (GtkWidget *widget)
+{
+        GPtrArray *env;
+        char      *str;
+        int        i;
+        static const char *allowed_env_vars [] = {
+                "PATH",
+                "SESSION_MANAGER",
+                "XAUTHORITY",
+                "XAUTHLOCALHOSTNAME",
+                "LANG",
+                "LANGUAGE"
+        };
+
+        env = g_ptr_array_new ();
+
+        str = gdk_screen_make_display_name (gtk_widget_get_screen (widget));
+        g_ptr_array_add (env, g_strdup_printf ("DISPLAY=%s", str));
+        g_free (str);
+
+        g_ptr_array_add (env, g_strdup_printf ("HOME=%s",
+                                               g_get_home_dir ()));
+
+        for (i = 0; i < G_N_ELEMENTS (allowed_env_vars); i++) {
+                const char *var;
+                const char *val;
+                var = allowed_env_vars [i];
+                val = g_getenv (var);
+                if (val != NULL) {
+                        g_ptr_array_add (env, g_strdup_printf ("%s=%s",
+                                                               var,
+                                                               val));
+                }
+        }
+
+        str = widget_get_id_string (widget);
+        g_ptr_array_add (env, g_strdup_printf ("XSCREENSAVER_WINDOW=%s", str));
+        g_free (str);
+
+        g_ptr_array_add (env, NULL);
+
+        return env;
+}
+
 static gboolean
 spawn_on_widget (GtkWidget  *widget,
                  const char *command,
@@ -612,7 +623,6 @@ spawn_on_widget (GtkWidget  *widget,
         char       *path;
         char      **argv;
         GPtrArray  *env;
-        char       *str;
         gboolean    result;
         GIOChannel *channel;
         GError     *error = NULL;
@@ -621,10 +631,15 @@ spawn_on_widget (GtkWidget  *widget,
         int         id;
         int         i;
 
-        if (! command)
+        if (! command) {
                 return FALSE;
+        }
 
-        g_shell_parse_argv (command, NULL, &argv, NULL);
+        if (! g_shell_parse_argv (command, NULL, &argv, &error)) {
+                g_warning ("Could not parse command: %s", error->message);
+                g_error_free (error);
+                return FALSE;
+        }
 
         path = find_command (argv [0]);
         if (path) {
@@ -632,33 +647,11 @@ spawn_on_widget (GtkWidget  *widget,
                 argv [0] = path;
         }
 
-        env = g_ptr_array_new ();
-
-        str = widget_get_id_string (widget);
-        g_ptr_array_add (env, g_strdup_printf ("XSCREENSAVER_WINDOW=%s", str));
-        g_free (str);
-
-        str = gdk_screen_make_display_name (gtk_widget_get_screen (widget));
-        g_ptr_array_add (env, g_strdup_printf ("DISPLAY=%s", str));
-        g_free (str);
-
-        g_ptr_array_add (env, g_strdup_printf ("HOME=%s",
-                                               g_get_home_dir ()));
-        g_ptr_array_add (env, g_strdup_printf ("PATH=%s", g_getenv ("PATH")));
-
-        if (g_getenv ("XAUTHORITY"))
-                g_ptr_array_add (env, g_strdup_printf ("XAUTHORITY=%s",
-                                                       g_getenv ("XAUTHORITY")));
-        if (g_getenv ("LANG"))
-                g_ptr_array_add (env, g_strdup_printf ("LANG=%s",
-                                                       g_getenv ("LANG")));
-        if (g_getenv ("LANGUAGE"))
-                g_ptr_array_add (env, g_strdup_printf ("LANGUAGE=%s",
-                                                       g_getenv ("LANGUAGE")));
-        g_ptr_array_add (env, NULL);
+        env = get_env_vars (widget);
 
+        error = NULL;
         result = gdk_spawn_on_screen_with_pipes (gtk_widget_get_screen (widget),
-                                                 g_get_home_dir (),
+                                                 NULL,
                                                  argv,
                                                  (char **)env->pdata,
                                                  G_SPAWN_DO_NOT_REAP_CHILD,
@@ -669,12 +662,13 @@ spawn_on_widget (GtkWidget  *widget,
                                                  NULL,
                                                  &standard_error,
                                                  &error);
-        for (i = 0; i < env->len; i++)
+        for (i = 0; i < env->len; i++) {
                 g_free (g_ptr_array_index (env, i));
+        }
         g_ptr_array_free (env, TRUE);
 
         if (! result) {
-                g_warning ("Could not start command '%s': %s", argv [0], error->message);
+                g_warning ("Could not start command '%s': %s", command, error->message);
                 g_error_free (error);
                 g_strfreev (argv);
                 return FALSE;
@@ -684,10 +678,11 @@ spawn_on_widget (GtkWidget  *widget,
 
         nice_process (child_pid, 10);
 
-        if (pid)
+        if (pid) {
                 *pid = child_pid;
-        else
+        } else {
                 g_spawn_close_pid (child_pid);
+        }
 
         channel = g_io_channel_unix_new (standard_error);
         g_io_channel_set_close_on_unref (channel, TRUE);
@@ -698,8 +693,9 @@ spawn_on_widget (GtkWidget  *widget,
                              G_IO_IN | G_IO_HUP | G_IO_ERR | G_IO_NVAL,
                              watch_func,
                              user_data);
-        if (watch_id)
+        if (watch_id) {
                 *watch_id = id;
+        }
 
         g_io_channel_unref (channel);
 
@@ -721,6 +717,7 @@ command_watch (GIOChannel   *source,
                 char *str;
 
                 status = g_io_channel_read_line (source, &str, NULL, NULL, &error);
+
 		if (status == G_IO_STATUS_NORMAL) {
 		} else if (status == G_IO_STATUS_EOF) {
                         done = TRUE;
@@ -741,6 +738,18 @@ command_watch (GIOChannel   *source,
         return TRUE;
 }
 
+gboolean
+gs_job_is_running (GSJob *job)
+{
+        gboolean running;
+
+        g_return_val_if_fail (GS_IS_JOB (job), FALSE);
+
+        running = (job->priv->pid > 0);
+
+        return running;
+}
+
 gboolean
 gs_job_start (GSJob *job)
 {
@@ -780,28 +789,39 @@ gs_job_start (GSJob *job)
                                   job,
                                   &job->priv->watch_id);
 
-        if (result)
+        if (result) {
                 job->priv->status = GS_JOB_RUNNING;
+        }
 
         gs_job_theme_info_unref (info);
 
         return result;
 }
 
+static void
+remove_command_watch (GSJob *job)
+{
+        if (job->priv->watch_id != 0) {
+                g_source_remove (job->priv->watch_id);
+                job->priv->watch_id = 0;
+        }
+}
+
 gboolean
 gs_job_stop (GSJob *job)
 {
         g_return_val_if_fail (job != NULL, FALSE);
         g_return_val_if_fail (GS_IS_JOB (job), FALSE);
 
-        if (! job->priv->pid)
+        if (! job->priv->pid) {
                 return FALSE;
+        }
 
-        if (job->priv->status == GS_JOB_STOPPED)
+        if (job->priv->status == GS_JOB_STOPPED) {
                 gs_job_suspend (job, FALSE);
+        }
 
-        g_source_remove (job->priv->watch_id);
-        job->priv->watch_id = 0;
+        remove_command_watch (job);
 
         signal_pid (job->priv->pid, SIGTERM);
 
@@ -819,8 +839,9 @@ gs_job_suspend (GSJob   *job,
         g_return_val_if_fail (job != NULL, FALSE);
         g_return_val_if_fail (GS_IS_JOB (job), FALSE);
 
-        if (! job->priv->pid)
+        if (! job->priv->pid) {
                 return FALSE;
+        }
 
         signal_pid (job->priv->pid, (suspend ? SIGSTOP : SIGCONT));
 
