@@ -20,9 +20,6 @@
  *
  */
 
-/* DPMS support has been moved to gnome-power-manager */
-#undef USE_LEGACY_DPMS_SUPPORT
-
 #include "config.h"
 #include <stdlib.h>
 #include <stdio.h>
@@ -38,10 +35,6 @@
 #include "gs-watcher.h"
 #include "gs-fade.h"
 
-#ifdef USE_LEGACY_DPMS_SUPPORT
-#include "gs-power.h"
-#endif
-
 #include "gs-listener-dbus.h"
 #include "gs-monitor.h"
 #include "gs-prefs.h"
@@ -58,9 +51,6 @@ struct GSMonitorPrivate
         GSWatcher      *watcher;
         GSListener     *listener;
         GSManager      *manager;
-#ifdef USE_LEGACY_DPMS_SUPPORT
-        GSPower        *power;
-#endif
         GSPrefs        *prefs;
         GSFade         *fade;
 };
@@ -100,6 +90,20 @@ manager_deactivated_cb (GSManager *manager,
         gs_listener_set_active (monitor->priv->listener, FALSE);
 }
 
+static void
+manager_auth_request_begin_cb (GSManager *manager,
+                               GSMonitor *monitor)
+{
+        gs_listener_emit_auth_request_begin (monitor->priv->listener);
+}
+
+static void
+manager_auth_request_end_cb (GSManager *manager,
+                             GSMonitor *monitor)
+{
+        gs_listener_emit_auth_request_end (monitor->priv->listener);
+}
+
 static gboolean
 watcher_idle_cb (GSWatcher *watcher,
                  gboolean   is_idle,
@@ -159,17 +163,11 @@ listener_lock_cb (GSListener *listener,
                   GSMonitor  *monitor)
 {
         gboolean res;
-        gboolean locked;
 
-        /* set lock flag before trying to activate screensaver
-           in case something tries to react to the ActiveChanged signal */
-
-        gs_manager_get_lock_active (monitor->priv->manager, &locked);
-        gs_manager_set_lock_active (monitor->priv->manager, TRUE);
         res = gs_listener_set_active (monitor->priv->listener, TRUE);
-        if (! res) {
-                /* If we've failed then restore lock status */
-                gs_manager_set_lock_active (monitor->priv->manager, locked);
+        if (res) {
+                gs_manager_set_lock_active (monitor->priv->manager, TRUE);
+        } else {
                 gs_debug ("Unable to lock the screen");
         }
 }
@@ -205,17 +203,6 @@ listener_active_changed_cb (GSListener *listener,
                 goto done;
         }
 
-#ifdef USE_LEGACY_DPMS_SUPPORT
-                /* enable power management */
-                res = gs_power_set_active (monitor->priv->power, active);
-                if (! res) {
-                        gs_debug ("Unable to set power management active: %d", active);
-
-                        /* if we can't activate power management it isn't the
-                           end of the world */
-                }
-#endif
-
         ret = TRUE;
 
  done:
@@ -232,11 +219,11 @@ listener_active_changed_cb (GSListener *listener,
 }
 
 static void
-listener_throttled_changed_cb (GSListener *listener,
-                               gboolean    enabled,
-                               GSMonitor  *monitor)
+listener_throttle_changed_cb (GSListener *listener,
+                              gboolean    throttled,
+                              GSMonitor  *monitor)
 {
-        gs_manager_set_throttle_enabled (monitor->priv->manager, enabled);
+        gs_manager_set_throttled (monitor->priv->manager, throttled);
 }
 
 static void
@@ -247,38 +234,11 @@ listener_poke_cb (GSListener *listener,
            idle watcher */
         gs_watcher_reset (monitor->priv->watcher);
 
-#ifdef USE_LEGACY_DPMS_SUPPORT
-        /* turn on the monitor power */
-        gs_power_set_mode (monitor->priv->power,
-                           GS_POWER_MODE_ON);
-#endif
-
         /* request that the manager unlock -
            will pop up a dialog if necessary */
         gs_manager_request_unlock (monitor->priv->manager);
 }
 
-#ifdef USE_LEGACY_DPMS_SUPPORT
-static void
-power_changed_cb (GSPower    *power,
-                  GSPowerMode mode,
-                  GSMonitor  *monitor)
-{
-        gboolean is_on;
-
-        if (mode == GS_POWER_MODE_ON) {
-                is_on = TRUE;
-        } else {
-                is_on = FALSE;
-        }
-
-        /* Don't run themes if the monitor power is off */
-        if (! is_on) {
-                gs_manager_set_throttle_enabled (monitor->priv->manager, TRUE);
-        }
-}
-#endif
-
 static void
 _gs_monitor_update_from_prefs (GSMonitor *monitor,
                                GSPrefs   *prefs)
@@ -319,16 +279,6 @@ _gs_monitor_update_from_prefs (GSMonitor *monitor,
         if (activate_watch) {
                 gs_watcher_set_active (monitor->priv->watcher, TRUE);
         }
-
-
-#ifdef USE_LEGACY_DPMS_SUPPORT
-        gs_power_set_timeouts (monitor->priv->power,
-                               monitor->priv->prefs->dpms_standby,
-                               monitor->priv->prefs->dpms_suspend,
-                               monitor->priv->prefs->dpms_off);
-        gs_power_set_enabled (monitor->priv->power,
-                              monitor->priv->prefs->dpms_enabled);
-#endif
 }
 
 static void
@@ -338,7 +288,7 @@ disconnect_listener_signals (GSMonitor *monitor)
         g_signal_handlers_disconnect_by_func (monitor->priv->listener, listener_quit_cb, monitor);
         g_signal_handlers_disconnect_by_func (monitor->priv->listener, listener_cycle_cb, monitor);
         g_signal_handlers_disconnect_by_func (monitor->priv->listener, listener_active_changed_cb, monitor);
-        g_signal_handlers_disconnect_by_func (monitor->priv->listener, listener_throttled_changed_cb, monitor);
+        g_signal_handlers_disconnect_by_func (monitor->priv->listener, listener_throttle_changed_cb, monitor);
         g_signal_handlers_disconnect_by_func (monitor->priv->listener, listener_poke_cb, monitor);
 }
 
@@ -353,8 +303,8 @@ connect_listener_signals (GSMonitor *monitor)
                           G_CALLBACK (listener_cycle_cb), monitor);
         g_signal_connect (monitor->priv->listener, "active-changed",
                           G_CALLBACK (listener_active_changed_cb), monitor);
-        g_signal_connect (monitor->priv->listener, "throttle-enabled-changed",
-                          G_CALLBACK (listener_throttled_changed_cb), monitor);
+        g_signal_connect (monitor->priv->listener, "throttle-changed",
+                          G_CALLBACK (listener_throttle_changed_cb), monitor);
         g_signal_connect (monitor->priv->listener, "poke",
                           G_CALLBACK (listener_poke_cb), monitor);
 }
@@ -380,6 +330,8 @@ disconnect_manager_signals (GSMonitor *monitor)
 {
         g_signal_handlers_disconnect_by_func (monitor->priv->manager, manager_activated_cb, monitor);
         g_signal_handlers_disconnect_by_func (monitor->priv->manager, manager_deactivated_cb, monitor);
+        g_signal_handlers_disconnect_by_func (monitor->priv->manager, manager_auth_request_begin_cb, monitor);
+        g_signal_handlers_disconnect_by_func (monitor->priv->manager, manager_auth_request_end_cb, monitor);
 }
 
 static void
@@ -389,23 +341,12 @@ connect_manager_signals (GSMonitor *monitor)
                           G_CALLBACK (manager_activated_cb), monitor);
         g_signal_connect (monitor->priv->manager, "deactivated",
                           G_CALLBACK (manager_deactivated_cb), monitor);
+        g_signal_connect (monitor->priv->manager, "auth-request-begin",
+                          G_CALLBACK (manager_auth_request_begin_cb), monitor);
+        g_signal_connect (monitor->priv->manager, "auth-request-end",
+                          G_CALLBACK (manager_auth_request_end_cb), monitor);
 }
 
-#ifdef USE_LEGACY_DPMS_SUPPORT
-static void
-disconnect_power_signals (GSMonitor *monitor)
-{
-        g_signal_handlers_disconnect_by_func (monitor->priv->power, power_changed_cb, monitor);
-}
-
-static void
-connect_power_signals (GSMonitor *monitor)
-{
-        g_signal_connect (monitor->priv->power, "changed",
-                          G_CALLBACK (power_changed_cb), monitor);
-}
-#endif
-
 static void
 disconnect_prefs_signals (GSMonitor *monitor)
 {
@@ -438,11 +379,6 @@ gs_monitor_init (GSMonitor *monitor)
         monitor->priv->manager = gs_manager_new ();
         connect_manager_signals (monitor);
 
-#ifdef USE_LEGACY_DPMS_SUPPORT
-        monitor->priv->power = gs_power_new ();
-        connect_power_signals (monitor);
-#endif
-
         _gs_monitor_update_from_prefs (monitor, monitor->priv->prefs);
 }
 
@@ -461,18 +397,12 @@ gs_monitor_finalize (GObject *object)
         disconnect_watcher_signals (monitor);
         disconnect_listener_signals (monitor);
         disconnect_manager_signals (monitor);
-#ifdef USE_LEGACY_DPMS_SUPPORT
-        disconnect_power_signals (monitor);
-#endif
         disconnect_prefs_signals (monitor);
 
         g_object_unref (monitor->priv->fade);
         g_object_unref (monitor->priv->watcher);
         g_object_unref (monitor->priv->listener);
         g_object_unref (monitor->priv->manager);
-#ifdef USE_LEGACY_DPMS_SUPPORT
-        g_object_unref (monitor->priv->power);
-#endif
         g_object_unref (monitor->priv->prefs);
 
         G_OBJECT_CLASS (parent_class)->finalize (object);
